#3D golf game
from scene import *
import math
#move angle
def moveangle(x,y,deg,speed):
	x += math.cos(math.radians(deg))*speed
	y += math.sin(math.radians(deg))*speed
	return x,y
#calculate distance and degrees
def getdist(xa,ya,xb,yb):
	dx = xa-xb
	dy = ya-yb
	tempdeg = math.degrees(math.atan2(dy,dx))
	tempdist = math.sqrt(dx*dx+dy*dy)
	return(tempdist,tempdeg)
#3D to 2D
def getxy(x,y,z,cx,cy,cdeg,w,h):
	if (x<0) or (x>49) or (y<0) or (y>49) :return (-999,-999)
	dist,deg = getdist(x,y,cx,cy)
	deg = deg-cdeg
	if deg>180 :deg -= 360
	if deg<-180 :deg += 360
	if (abs(deg)>60): return(-999,-999)
	if dist==0:dist = 0.01
	gx = (deg*(w/2)/45)+(w/2)
	gy = (h/2)+(z*120/dist)
	return(gx,h-gy)
#scene
class MyScene(Scene):
	def setup(self):
		#map
		self.mapdata = [[0 for i in range(50)] for j in range(50)]
		for y in range(50):
			for x in range(50):
				z = math.cos(math.radians(x*30))+math.cos(math.radians(y*40))
				self.mapdata[x][y] = z

		self.cdeg = 0  #camera angle
		self.cx = 0	#camera position
		self.cy = 0
		self.cz = 0
		self.bx = 2  #ball position
		self.by = 24
		self.bz = 0
		self.bdeg = 0   #ball angle
		self.ex = 40  #cup position
		self.ey = 26
		self.ez = self.mapdata[self.ex][self.ey]
		self.tx = -1 #touch position
		self.ty = -1
		self.seq = 0
		self.msg = ''

	def update(self):
		w = self.size.w
		h = self.size.h
		fill('blue')
		rect(0,h/2,w,h/2)
		fill('green')
		rect(0,0,w,h/2)				
		#draw map
		stroke('lime')
		stroke_weight(1)
		for mode in range(2):
			for j in range(50):
				x2 = -999
				y2 = -999
				for i in range(50):
					if mode:
						x = i
						y = j
					else:
						x = j
						y = i

					z = self.mapdata[x][y]-self.cz
					x1,y1 = getxy(x,y,z,self.cx,self.cy,self.cdeg,w,h)
					if (x1 > -999) and (x2 > -999):line(x1,y1,x2,y2)
					x2 = x1
					y2 = y1

		z = self.bz-self.cz
		gx,gy = getxy(self.bx,self.by,z,self.cx,self.cy,self.cdeg,w,h)
		fill('white')
		ellipse(gx-5, gy-5, 10, 10)   #ball
		z = self.ez-self.cz
		gx,gy = getxy(self.ex,self.ey,z,self.cx,self.cy,self.cdeg,w,h)
		fill('red')
		ellipse(gx-5, gy-5, 10, 10)     #cup	
		if self.seq==0:
			self.dist,self.cdeg = getdist(self.ex,self.ey,self.bx,self.by)
			self.bz = self.mapdata[int(self.bx)][int(self.by)]
			self.bspeed = 0
			self.bz1 = 0
			self.seq += 1
		elif self.seq==1:
			self.cx,self.cy = moveangle(self.bx,self.by,self.cdeg+180,3)
			self.cz = self.bz-2
			self.bspeed += 0.05
			if (self.bspeed > 1): self.bspeed=0
			fill('red')
			rect(w/8,h/4,self.bspeed*100,20)
			self.msg = "TAP SCREEN"
			if self.tx!=-1:
				if self.tx<(w/3):
					self.cdeg -= 2
				elif self.tx>(w*2/3):
					self.cdeg += 2
				else:
					self.bdeg = self.cdeg
					self.bz1 = -self.bspeed
					self.cx,self.cy = moveangle(self.bx,self.by,self.bdeg+60,15)
					self.dist,self.cdeg = getdist(self.bx,self.by,self.cx,self.cy)
					self.seq += 1

			self.tx = -1
			self.ty = -1
		elif self.seq==2:
			self.bx,self.by = moveangle(self.bx,self.by,self.bdeg,self.bspeed)
			self.cx,self.cy = moveangle(self.cx,self.cy,self.bdeg,self.bspeed)
			if (self.bx<0) or (self.by<0) or (self.bx>49) or (self.by>49):
				self.msg = "OB"
				self.seq += 1

			gz = self.mapdata[int(self.bx)][int(self.by)]
			if (gz < (self.bz+self.bz1)):self.bz1 *= -0.7
			self.bspeed *= 0.96
			self.bz += self.bz1
			self.bz1 += 0.05
			self.dist,self.deg = getdist(self.bx,self.by,self.ex,self.ey)
			if (self.dist<0.4) and (abs(self.bspeed)<0.1):
				self.msg = "CUP IN"
				self.seq += 1
			elif (abs(self.bspeed)<0.02):
				self.bspeed=0
				self.seq = 0
		text(self.msg, font_size=24, x=w/2, y=h/10)		

	def touch_began(self,touch):
		self.tx,self.ty = touch.location

run(MyScene())
